package cz.vsb.jtestserver.service;

import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.NamingException;
import javax.naming.directory.InitialDirContext;

import org.apache.log4j.Logger;

import cz.vsb.jtestserver.IConstants;
import cz.vsb.jtestserver.util.Configuration;
import cz.vsb.jtestserver.util.Database;
import cz.vsb.jtestserver.view.UserView;
import java.sql.PreparedStatement;

/**
 * Autentizace a autorizace uzivatelu.
 * 
 * @author Martin Lason
 */
public class SecurityService implements IAuthentication {

	static Logger logger = Logger.getLogger(SecurityService.class);

	/**
	 * Prihlaseni uzivatele do systemu
	 * 
	 * @param username uzivatelske jmeno
	 * @param password heslo
	 * @return informace o prihlasenem uzivateli
	 */
	public UserView login(String username, String password)
			throws InvalidLoginException {
		if (authLDAP(buildLdapDN(username), password)) {
			return getUserView(username);
		} else if (authMySQL(username, password)) {
			return getUserView(username);
		} else {
			throw new InvalidLoginException("Neautorizovany pristup");
		}
	}

	/**
	 * Autentizace proti LDAP serveru.
	 * 
	 * @param bindDN prihlasovaci jmeno vcetne kontextu (DN)
	 * @param password heslo
	 * @return true pokud uzivatel existuje a bylo spravne zadano heslo,
	 * 		jinak false
	 */
	private boolean authLDAP(String bindDN, String password) {
		// Set up the environment for creating the initial context
		Hashtable<String, String> env = new Hashtable<String, String>();
		env.put(Context.INITIAL_CONTEXT_FACTORY,
				"com.sun.jndi.ldap.LdapCtxFactory");
		env.put(Context.PROVIDER_URL, Configuration.getProperty("ldapurl"));

		env.put(Context.SECURITY_AUTHENTICATION, "simple");
		env.put(Context.SECURITY_PRINCIPAL, bindDN);
		env.put(Context.SECURITY_CREDENTIALS, password);

		try {
			new InitialDirContext(env);
		} catch (NamingException e) {
			logger.warn("Neuspesne prihlaseni uzivatele " + bindDN + " do LDAP");
			return false;
		}
		logger.debug("Uzivatel " + bindDN + " prihlasen pres LDAP");
		return true;
	}

	private boolean authMySQL(String username, String password) {
		Connection con = null;
		
		String dbPass = null;
                PreparedStatement sql = null;
                ResultSet result = null;
		try {
			con = Database.getConnection();			
			sql = con.prepareStatement("SELECT pass FROM auth WHERE login =?");
                        sql.setString(1,username);                        
                        logger.debug(sql.toString());
			result = sql.executeQuery();
			if (result.next()) {
				dbPass = result.getString("pass");
			} else {
				logger.warn("MySQL: neznamy uzivatel " + username);
				return false;
			}
		} catch (SQLException e) {
			logger.error(e, e);
		} finally {
                  try {
                    result.close();
                    sql.close();
                    con.close();
                  } catch (SQLException ex) {
                    logger.error(ex,ex);
                  }
                }

		String md5Pass = null;
		try {
			MessageDigest md5 = MessageDigest.getInstance("MD5");
			md5.update(password.getBytes());
			BigInteger hash = new BigInteger(1, md5.digest());
			md5Pass = hash.toString(16);
		} catch (NoSuchAlgorithmException ns) {
			logger.error(ns, ns);
			return false;
		}
		if (dbPass != null && md5Pass != null && dbPass.equals(md5Pass)) {
			logger.debug("Uzivatel " + username + " prihlasen pres MySQL");
			return true;
		} else {
			logger.warn("Neuspesne prihlaseni uzivatele " + username 
					+ " do MySQL");
			return false;
		}
	}
	
	/**
	 * Sestaveni DN pro LDAP - vytvoreni kontextu
	 * 
	 * @param cn prihlasovaci jmeno
	 * @return DN jmeno
	 */
	private String buildLdapDN(String cn) {
		StringBuilder sb = new StringBuilder(255);
		sb.append("cn=");
		sb.append(cn);
		sb.append(",o=");
		sb.append(cn.charAt(cn.length() - 1));
		return sb.toString();
	}
	
	/**
	 * Autorizace uzivatele. 
	 * Vyhledani informaci o uzivateli v MySQL databazi.
	 * 
	 * @param username prihlasovaci jmeno
	 * @return informace o uzivateli
	 * @throws InvalidLoginException 
	 */
	private UserView getUserView(String username) throws InvalidLoginException {
		if (username.equals("root")) {
			UserView uview = new UserView();
			uview.setLogin("root");
			uview.setStatus(IConstants.STATUS_ROOT);
			return uview;
		} else {
			Connection con = null;
			try {
				// Vyhledani informaci o osobe
				con = Database.getConnection();
				String query = "SELECT a.login, tit_pred, jmeno, prijmeni, tit_za, status "
					+ "FROM auth a, status s WHERE a.login = s.login AND a.login = '"
					+ username + '\'';
				logger.debug(query);
				Statement sql = con.createStatement();
				ResultSet result = sql.executeQuery(query);
				UserView uview = new UserView();
				if (result.next()) {
					uview.setLogin(result.getString(1));
					uview.setTitulPred(result.getString("tit_pred"));
					uview.setJmeno(result.getString("jmeno"));
					uview.setPrijmeni(result.getString("prijmeni"));
					uview.setTitulZa(result.getString("tit_za"));
					switch (result.getInt("status")) {
					case IConstants.STATUS_ROOT_CODE:
						uview.setStatus(IConstants.STATUS_ROOT);
						break;
					case IConstants.STATUS_UCITEL_CODE:
						uview.setStatus(IConstants.STATUS_UCITEL);
						break;
					case IConstants.STATUS_STUDENT_CODE:
						uview.setStatus(IConstants.STATUS_STUDENT);
						break;
					default:
						uview.setStatus(IConstants.STATUS_STUDENT);
					}
				} else {
					logger.warn("Chyba autorizace: neznamy uzivatel");
					throw new InvalidLoginException("Neznamy uzivatel");
				}
                                result.close();
                                sql.close();
                                con.close();
				return uview;
			} catch (SQLException e) {
				logger.error(e, e);
			}
		}
		return null;
	}
}
